class ArticleStatus:
    ACTIVE = "active"
    UNLISTED = "unlisted"
    DRAFT = "draft"

class CacheTypes:
    NEW_FILE = "NEW_FILE"
    META_CHANGE = "META_CHANGE"
    CONTENT_CHANGE = "CONTENT_CHANGE"
    REMOVED_FILE = "REMOVED_FILE"